﻿
namespace Common.IdentityServerRepository.MySQL
{
    using Microsoft.AspNetCore.Identity.EntityFrameworkCore;
    using Microsoft.EntityFrameworkCore;    
    using Common.IdentityServerRepository.Interfaces;
    using Entities;
    using Converters;

    public class IdentityServerDbContext :
        IdentityDbContext<MyUser, MyRole, string>,
        IIdentityServerDbContext<MyClient, MySecret, MyClientSecret, string>        
    {
        private readonly DbContextOptions _options;

        public IdentityServerDbContext(DbContextOptions options)
            : base(options)
        {
            _options = options;
        }        
        public virtual DbSet<MyClient> Clients { get; set; }        
        public virtual DbSet<MyClientSecret> ClientSecrets { get; set; }
        public virtual DbSet<MySecret> Secrets { get; set; }
        protected override void OnModelCreating(ModelBuilder builder)
        {
            //Configures the schema needed for the identity framework
            //for Users, Roles and Claims
            base.OnModelCreating(builder);

            //Configures the schema needed for the identity framework
            //for custom extended entities
            builder.Entity<MyClient>(b =>
            {
                b.HasKey(pk => pk.Id);
                b.HasIndex(idx => idx.ClientName).HasName("idx_clientname");
                b.HasIndex(idx => idx.UserId).HasName("idx_userid");
                b.HasMany<MyClientSecret>().WithOne().HasForeignKey(fk => fk.ClientId).IsRequired();
                b.Property(p => p.LockoutEnabled).HasColumnType("BIT(1)");//.HasConversion(new BoolToShortConverter());                
                b.Property(p => p.Id).HasColumnType("VARCHAR(256)");
                b.Property(p => p.UserId).HasColumnType("VARCHAR(256)");
                b.Property(p => p.ClientName).HasColumnType("VARCHAR(256)");
                b.Property(p => p.Description).HasColumnType("TEXT");
                b.ToTable("AspNetClients");
            });

            builder.Entity<MySecret>(b =>
            {
                b.HasKey(pk => pk.Id);
                b.HasMany<MyClientSecret>().WithOne().HasForeignKey(fk => fk.SecretId).IsRequired();
                b.Property(p => p.Id).HasColumnType("VARCHAR(256)");
                b.Property(p => p.Description).HasColumnType("TEXT");
                b.Property(p => p.Value).HasColumnType("VARCHAR(256)");
                b.Property(p => p.Type).HasColumnType("VARCHAR(32)");
                b.ToTable("AspNetSecrets");
            });
                       
            builder.Entity<MyClientSecret>(b =>
            {
                b.HasKey(pk => new { pk.ClientId, pk.SecretId });
                b.Property(p => p.ClientId).HasColumnType("VARCHAR(256)");
                b.Property(p => p.SecretId).HasColumnType("VARCHAR(256)");
                b.ToTable("AspNetClientSecrets");
            });

            builder.Entity<MyUser>(b =>
            {
                b.Property(p => p.EmailConfirmed).HasConversion(new BoolToShortConverter());
                b.Property(p => p.LockoutEnabled).HasConversion(new BoolToShortConverter());
                b.Property(p => p.PhoneNumberConfirmed).HasConversion(new BoolToShortConverter());
                b.Property(p => p.TwoFactorEnabled).HasConversion(new BoolToShortConverter());
            });
        }
    }
}
