﻿
namespace Common.IdentityServerRepository.MySQL
{   
    using System;
    using Microsoft.Extensions.DependencyInjection;    
    using Common.IdentityServerRepository.Infrastructure;
    using Common.IdentityServerRepository.Stores;
    using Entities;
    using System.Threading.Tasks;
    using Microsoft.AspNetCore.Identity;
    using System.Linq;

    public class MyClientManager : ClientManager<MyClient, string>, IDisposable
    {
        private readonly SecretStore<MySecret, string> _secretStore;
        private readonly ClientSecretStore<MyClientSecret, string> _clientSecretStore;
        private SecretStore<MySecret, string> SecretStore => _secretStore;
        private ClientSecretStore<MyClientSecret, string> ClientSecretStore => _clientSecretStore;
        
        public MyClientManager(ClientStore<MyClient, string> store, IServiceProvider services) :
            base(store, services)
        {
            _secretStore = Services.GetService<SecretStore<MySecret, string>>();
            _clientSecretStore = Services.GetService<ClientSecretStore<MyClientSecret, string>>();
        }

        public static void Create(IServiceCollection services)
        {            
            services.AddScoped<ClientStore<MyClient, string>>();
            services.AddScoped<SecretStore<MySecret, string>>();
            services.AddScoped<ClientSecretStore<MyClientSecret, string>>();
            services.AddScoped<MyClientManager>();                    
        }

        public async Task<bool> ExistsAsync(string clientId)
        {   
            return await Task.FromResult(Clients.Any(a => a.Id == clientId));
        }
        public async Task<IdentityResult> AddSecretAsync(string clientId, MySecret secret)
        {
            SecretStore.Add(secret);
            ClientSecretStore.Add(new MyClientSecret { ClientId = clientId, SecretId = secret.Id });
            await Store.SaveAllChangesAsync();
            return IdentityResult.Success;
        }
        public async Task<IdentityResult> AddSecretAsync(MyClient client, MySecret secret)
        {
            Store.Add(client);
            SecretStore.Add(secret);
            ClientSecretStore.Add(new MyClientSecret { ClientId = client.Id, SecretId = secret.Id });
            await Store.SaveAllChangesAsync();
            return IdentityResult.Success;
        }
    }
}
